@@ -69,6 +69,12 @@ class ColumnGroup:
     columns: List[str]
 
 
+@dataclass
+class GridTotals:
+    page: Optional[Dict[str, Any]] = None
+    grand: Optional[Dict[str, Any]] = None
+
+
 @dataclass
 class GridSettings:
     search_expr: Optional[str] = None
@@ -150,4 +156,5 @@ class Grid:
     spec: GridSpec
     state: GridState
     records: List[Dict[str, Any]]
+    totals: GridTotals
     errors: List[str]
